# Copyright 2022 Yu-Kai Lin. All rights reserved.
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file.

import click.shell_completion

from click_pwsh.shell_completion import PwshComplete

_registered = False


def support_pwsh_shell_completion() -> None:
    """Support shell completion for PowerShell 7."""
    global _registered

    if not _registered:
        click.shell_completion.add_completion_class(PwshComplete)
        _registered = True



# Copyright 2022 Yu-Kai Lin. All rights reserved.
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file.

import subprocess as sp
from pathlib import Path

import click


@click.group()
def main():
    pass


@main.command()
@click.argument("command")
def install(command):
    """Land the shell completion to PowerShell 7."""
    profile = (
        sp.run('pwsh -c "echo $PROFILE"', shell=True, capture_output=True)
        .stdout.decode()
        .strip()
    )
    profile = Path(profile)

    # Write the completion script to a local profile
    completion_varname = "_{}_COMPLETE".format(command.replace("-", "_").upper())

    completion_profile = profile.parent / ".{}_profile.ps1".format(command)
    sp.run(
        "pwsh -c \"$env:{0} = 'pwsh_source'; {1} > {2}; $env:{0} = $null\"".format(
            completion_varname, command, str(completion_profile)
        ),
        shell=True,
    )
    sp.run(
        'pwsh -c "echo \'"{}" | Invoke-Expression\' >> {}"'.format(
            str(completion_profile), str(profile)
        ),
        shell=True,
    )

    print("Complete.")


@main.command()
@click.argument("command")
def update(command):
    """Update shell completion scripts to PowerShell 7."""
    profile = (
        sp.run('pwsh -c "echo $PROFILE"', shell=True, capture_output=True)
        .stdout.decode()
        .strip()
    )
    profile = Path(profile)

    # Write the completion script to a local profile
    completion_varname = "_{}_COMPLETE".format(command.replace("-", "_").upper())

    completion_profile = profile.parent / ".{}_profile.ps1".format(command)

    if not completion_profile.exists():
        print("ERROR: Cannot find existing completion profile. Try `install` instead.")
        exit(1)

    sp.run(
        "pwsh -c \"$env:{0} = 'pwsh_source'; {1} > {2}; $env:{0} = $null\"".format(
            completion_varname, command, str(completion_profile)
        ),
        shell=True,
    )

    print("Complete.")


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['click_pwsh']

package_data = \
{'': ['*']}

install_requires = \
['click>=8.0.0,<9.0.0']

setup_kwargs = {
    'name': 'click-pwsh',
    'version': '0.9.3',
    'description': 'A click extension to support shell completion for PowerShell 7',
    'long_description': "# `PS>` click-pwsh\n\n[![Supported Python Versions](https://img.shields.io/pypi/pyversions/click-pwsh)](https://pypi.org/project/click-pwsh/) [![PyPI version](https://badge.fury.io/py/click-pwsh.svg)](https://badge.fury.io/py/click-pwsh)\n\nA [click](https://github.com/pallets/click) extension to support shell completion for **[PowerShell 7](https://github.com/PowerShell/PowerShell)**.\n\nThis extension is written based on click **8.x** (i.e., the rewritten click's completion system). Be aware of your click version before using it.\n\nHope it can provide smooth experiences for Windows users. d(`･∀･)b\n\n## Installation\n\nYou can get the package from PyPI:\n\n```bash\nPS> pip install click-pwsh\n```\n\n## Quickstart\n\nAdd the following code at the top of your script:\n\n```python\nfrom click_pwsh import support_pwsh_shell_completion\nsupport_pwsh_shell_completion()\n```\n\nAnd run the following command to install the shell completion:\n\n```bash\nPS> python -m click_pwsh install foo-bar\nComplete.\n```\n\nwhere `foo-bar` is your command name.\n\nThen ... all done. Re-open PowerShell 7 and enjoy the shell completion!\n\n## Update Shell Completion Scripts\n\nIf you upgrade click-pwsh, you can use the following command to update your shell completion scripts:\n\n```bash\nPS> python -m click_pwsh update foo-bar\nComplete.\n```\n\nwhere `foo-bar` is your command name whose shell completion scripts have already installed before.\n",
    'author': 'Yu-Kai Lin',
    'author_email': 'stephen359595@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/StephLin/click-pwsh',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)



# Copyright 2022 Yu-Kai Lin. All rights reserved.
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file.

import os
import typing as t

from click.parser import split_arg_string
from click.shell_completion import CompletionItem, ShellComplete

_SOURCE_PWSH = """\
Register-ArgumentCompleter -Native -CommandName %(prog_name)s -ScriptBlock {
    param($wordToComplete, $commandAst, $cursorPosition)
        $env:COMP_WORDS = $commandAst
        $env:COMP_WORDS = $env:COMP_WORDS.replace('\\', '/')
        $imcompleteCommand = $commandAst.ToString()

        $myCursorPosition = $cursorPosition
        if ($myCursorPosition -gt $imcompleteCommand.Length) { $myCursorPosition = \
$imcompleteCommand.Length }
        $env:COMP_CWORD = @($imcompleteCommand.substring(0, \
$myCursorPosition).Split(" ") | Where-Object { $_ -ne "" }).Length
        if ( $wordToComplete.Length -gt 0) { $env:COMP_CWORD -= 1 }

        $env:%(complete_var)s = "pwsh_complete"

        %(prog_name)s | ForEach-Object {
            $type, $value, $help = $_.Split(",", 3)
            if ( ($type -eq "plain") -and ![string]::IsNullOrEmpty($value) ) {
                [System.Management.Automation.CompletionResult]::new($value, $value, \
"ParameterValue", $value)
            } elseif ( ($type -eq "file") -or ($type -eq "dir") ) {
                if ([string]::IsNullOrEmpty($wordToComplete)) {
                    $dir = "./"
                } else {
                    $dir = $wordToComplete.replace('\\', '/')
                }
                Get-ChildItem -Path $dir | Resolve-Path -Relative | ForEach-Object {
                    $path = $_.ToString().replace('\\', '/')
                    if ((Get-Item $path) -is [System.IO.DirectoryInfo]) { $path = \
$path + "/" }
                    [System.Management.Automation.CompletionResult]::new($path, $path, \
"ParameterValue", $path)
                }
            }
        }

        $env:COMP_WORDS = $null | Out-Null
        $env:COMP_CWORD = $null | Out-Null
        $env:%(complete_var)s = $null | Out-Null
}
"""


class PwshComplete(ShellComplete):
    """Shell completion for PowerShell 7."""

    name = "pwsh"
    source_template = _SOURCE_PWSH

    def get_completion_args(self) -> t.Tuple[t.List[str], str]:
        cwords = split_arg_string(os.environ["COMP_WORDS"])
        cword = int(os.environ["COMP_CWORD"])
        args = cwords[1:cword]

        try:
            incomplete = cwords[cword]
        except IndexError:
            incomplete = ""

        return args, incomplete

    def format_completion(self, item: CompletionItem) -> str:
        return f"{item.type},{item.value},{item.help if item.help else '_'}"
